#include "HtmlPage.h"

/* Static members */
bool HtmlPage::initdone_ = false;
std::string HtmlPage::init_ = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">";
std::string HtmlPage::procdate_ = "Not Available";
HtmlTag HtmlPage::head_("head");
HtmlTag HtmlPage::tail_("p");
//HtmlTag HtmlPage::title_("title"); 
HtmlTag HtmlPage::style_("style","","<!-- h1, h2 ,h3 { color: #000000; text-align: left; font-style: fixed; }\n body { bgcolor: #FFFFFF; font-style: fixed;  }\n link { color: #FF0000; }\n v:link { color: #00FF00; }\n a:link { color: #0000FF; }\n img { border: 0; }\n hr { noshade: noshade; } -->\n");
HtmlTag HtmlPage::meta_("meta","","");
char HtmlPage::fill_ = ' ';


/* Write out the page to an ostream */

std::ostream &operator<<(std::ostream& os, HtmlPage hp)
{ 

	/* Build <head> section */
	os << hp.init_ << std::endl;
	HtmlTag h("head");
	if ( hp.title_.size() ) 
		h << hp.title_;
	
	if ( hp.style_.size() ) 
		h << hp.style_;
	
	if ( hp.meta_.size() ) 
		h << hp.meta_;

	os <<  h << std::endl; 

	/*
	 * Add tail to end of body_ 
	 * The tail must appear inside the body tags
	 */
	HtmlTag cpybody(hp.body_);
	cpybody << hp.tail_;  
	os << cpybody;
		
	os << std::endl;
	return os; 
}


HtmlTag HtmlPage::XMMLogo() 
{

	HtmlTag anchor("a","name=\"top\"","&nbsp;");
	HtmlTag img("img","src=\"" + stripdir(stringParameter("logofile")) + "\" alt=\"XMM Survey Science Centre\"" , "");

	anchor << img;
	return anchor;
}

void HtmlPage::linkbar(const std::string page="") 
{

	 const string pages[] = {
		"obspage"
		,"ppspage"
		,"epxpage"
		,"omxpage"
		,"rgxpage"
		,"catpage"
		,""
	};

	const string text[] = {
		"OBS Summary"
		,"PPS Summary"
		,"EPIC Summary"
		,"OM Summary"
		,"RGS Summary"
		,"Catalogue Summary"
		,""
	};

	HtmlTag emptyrow("tr","align=\"center\"",HtmlTag("td","colspan=\"6\"","&nbsp;"));

	HtmlTag tr("tr","align=\"center\"");

	for(unsigned int i=0;i<6;i++) 
	{

		if(pages[i] == page ) 
		{
			tr << HtmlTag("td" , "width=\"15%\"" , text[i]);

		} else {

			tr << HtmlTag("td" 
				, "width=\"15%\"" 
				, HtmlTag( "a","href=\""+ stripdir(stringParameter(pages[i])) +"\"",text[i] )
			);
		}
		
	}
	
	HtmlTag bar("table","width=\"90%\" align=\"centre\" border=\"0\" cellspacing=\"0\" cellpadding=\"3\"");
	bar << emptyrow << tr << emptyrow;

	body_ << bar;
	return;
	
}

void HtmlPage::sub_title(strlist collect , std::string text)
{

	std::transform( collect.begin() , collect.end() , collect.begin() , toString_<std::string>() );
	
	body_ << HtmlTag("h1","", text);

	HtmlTag tr1("tr","align=\"left\"");
	HtmlTag tr2("tr","align=\"left\"");

	tr1 << HtmlTag("th","","Revolution");
	tr2 << HtmlTag("td","",collect[1]);

	tr1 << HtmlTag("th","","Target");
	tr2 << HtmlTag("td","",collect[4]);

	tr1 << HtmlTag("th","","Scheduled Length");
	toString_<double> diff;
	tr2 << HtmlTag("td","", diff( STime(collect[3]) - STime(collect[2]) )  );

	tr1 << HtmlTag("th","","Observer");
	tr2 << HtmlTag("td","",collect[5]);
	
	HtmlTag tbl("table","width=\"100%\" border=\"0\"");
	tbl << tr1 << tr2;
	body_ << tbl;

}

void HtmlPage::addTable(strlist title , strtable alldata , std::string anchor) 
{
	HtmlTag rtn("pre");
	
	if (alldata.begin() == alldata.end()) {
		rtn << HtmlTag("tt","","No data");
		return ;
	}

	const std::string sep(4,fill_);
	
	/* add missing titles as (1xfill) */
	if (title.size()  < alldata.size() ) 
		title.insert(title.end() , alldata.size() - title.size() , std::string(1,fill_) );

	/* 
	 * Unify column lengths
	 * The '-' title filler should be replaced with ' ' in the release version
	 * '-' is just that bit more obvious during testing.
	 */

	std::string::size_type lngth;
	
	for( strtable::size_type i = 0 ; i < alldata.size() ; i++ ) {

		if (title[i] == colName("DURATION")
		 || title[i] == altName("DURATION")
		 || title[i] == colName("NREXP")
		 || title[i] == altName("NREXP")
		 || title[i] == colName("PRIORITY")
		 || title[i] == altName("PRIORITY")
		 || title[i] == colName("OSW_ID")
		 || title[i] == altName("OSW_ID")
		 || title[i] == colName("SRC_NUM")
		 || title[i] == altName("SRC_NUM")
		 || title[i] == colName("RGSORDER")
		 || title[i] == altName("RGSORDER")
		) {			

			/* Right justlfy this column */
			unifylength<strlist::iterator> ul_right(std::string(1,fill_) 
				, title[i].length() 
				, 1
			);

			lngth = ul_right( alldata[i].begin() 
				, alldata[i].end() 
			) - title[i].length() ;
		
			/* pad title if column contents longer than name */
			if ( lngth ) title[i].insert( title[i].begin() , lngth ,  fill_ );


		} else {
		
			/* Left justlfy this column */
			unifylength<strlist::iterator> ul_left(std::string(1,fill_) 
				, title[i].length() 
			);
	
			lngth = ul_left( alldata[i].begin() 
				, alldata[i].end() 
			) - title[i].length() ;
		
			/* pad title if column contents longer than name */
			if ( lngth ) title[i].insert( title[i].end() , lngth ,  fill_ );


		}
	}


	/* Make links.  This is a bodge until I get an output column
	 * class sorted
	 * Have to do expid ahead of instrument or it meeses up the
	 * links.
	 */

	for( strtable::size_type i = 0 ; i < alldata.size() ; i++ ) {

		std::string ttl = stripws(title[i]);

		if ( ! anchor.size() && i && (ttl == colName("EXP_ID")
		 || ttl == altName("EXP_ID"))
		 
		){
			make_expid_link( alldata[i].begin() , alldata[i].end()
				,alldata[i-1].begin() , alldata[i-1].end() 
			);
		}
	}
	
	for( strtable::size_type i = 0 ; i < alldata.size() ; i++ ) {

		std::string ttl = stripws(title[i]);
		
		if ( ! anchor.size() && (ttl == colName("INSTURMENT")
		 || ttl == altName("INSTRUMENT"))
		){
			make_inst_link( alldata[i].begin() , alldata[i].end() );
			
		} else if (ttl == colName("FILENAME")
		 || ttl == altName("FILENAME")
		){
			make_name_link( alldata[i].begin() , alldata[i].end() );
		}
	}

	/* write it out to the page */
	strlist::const_iterator CI = title.begin();
	HtmlTag tt1("tt","",*(CI++),"","\n");
	for ( ; CI != title.end() ; CI++ ) tt1 << sep << *CI;

	//rtn << HtmlTag("h4","",tt1);	
	if (anchor.size()) {

		rtn << HtmlTag("a","name=\""+anchor+"\"" , HtmlTag("strong","",tt1));
	} else {
		rtn << HtmlTag("strong","",tt1);	

	}

	for(strlist::size_type i = 0 ; i < alldata[0].size() ; i++) {

		HtmlTag tt2("tt","",alldata[0][i],"","\n");
		for (strtable::size_type j = 1 ; j < alldata.size() ; j++) {

			tt2 << sep << alldata[j][i];
		}

		rtn << tt2;
	}

	body_ << rtn;
	return;


}

void HtmlPage::make_inst_link(strlist::iterator first_ , strlist::iterator last_)
{
	while (first_ != last_) {
		
		std::string inst = stripws(*first_);
		std::string page = whichPage(inst);

		if (page != thispage_) {

			unsigned int lngth = first_->size();
			std::string href = "href=\"" +  page + "#top\"";
			
			HtmlTag anch("a" , href , inst , "" , std::string( lngth - inst.size() , fill_));
			*first_  = anch.print();

		}

		first_++;
	}
}

void HtmlPage::make_expid_link( strlist::iterator first_ 
	, strlist::iterator last_ 
	, strlist::iterator instfirst_ 
	, strlist::iterator instlast_
	
) {	
	while (first_ != last_ && instfirst_ != instlast_) {
		
		std::string inst = stripws(*instfirst_);
		std::string page = whichPage(inst);
		
		unsigned int lngth = first_->size();
		std::string expid = stripws(*first_);
		
		std::string key(inst);
		key += expid;

		if ( hasAnchor(key) ) {

			std::string href;
			
			if (page == thispage_) {
				href = "href=\"#" 
					+ inst
					+ expid
					+ "\""
				;

			} else {
					
				href = "href=\"" 
					+  page 
					+ "#" 
					+ inst
					+ expid
					+ "\""
				;
			}

			HtmlTag anch("a" , href , expid , "" , std::string( lngth - expid.size() , fill_));
			*first_  = anch.print();
		}

		first_++;
		instfirst_++;
	}
}

void HtmlPage::make_name_link(strlist::iterator first_ , strlist::iterator last_)
{

	while (first_ != last_) {
		unsigned int lngth = first_->size();
		
		std::string name = stripws(*first_);

		if ( name.find(".PNG") != std::string::npos
		 || name.find(".HTM") != std::string::npos
		 || name.find(".PDF") != std::string::npos
		){
			std::string href = "href=\"" +  name + "\"";
			
			HtmlTag anch("a" , href , name , "" , std::string( lngth - name.size() , fill_));
			*first_  = anch.print();
		}
		first_++;
	}
}

