#ifndef PPSSUMM_HTMLPAGE
#define PPSSUMM_HTMLPAGE

#include <string>
#include <algorithm>
#include <functional>
#include <iterator>

#include "ppssumm_typedefs.h"
#include "PpssummStatic.h"
#include "HtmlTag.h"

#include "ExtendFitsColumn.h"
#include "string_algo.h"
#include "stripws.h"
#include "STime.h"

#include "param.h"
#include "Tracer.h"
#include "errstr.h"

/*
 * Alternate init_
 * "<?xml version=\"1.0\" encoding=\"UTF-8\"?> <!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\">"
 *
 */
class HtmlPage : public PpssummStatic
{

	/* WARNING
	 * title_ is what goes in the <title> tags.
	 */
	public:
	HtmlPage(std::string s) 
		: PpssummStatic()
		, body_("body")
		, title_("title")
	{ 
		thispage_ = stripdir(stringParameter(s));
		
		body_ << XMMLogo();
		body_ << HtmlTag("hr");

		if (! initdone_) {
			tail_ << HtmlTag("a","href=\"#top\"","Return to top.");
			tail_ << HtmlTag("br");
			tail_ << PpssummStatic::version()  ;
			tail_ << HtmlTag("br");
			tail_ << "Processing Date: " + HtmlPage::procdate_ ;
			HtmlPage::initdone_=1;
		}

	}

	static std::string procdate_;

	HtmlTag body_;

	HtmlTag title() const
	{ return title_; }

	HtmlTag& title(std::string s)
	{ title_ << s ; return title_; }

	HtmlTag& tail() const
	{ return tail_; }

	HtmlTag& tail(std::string s)
	{ tail_ << s ; return tail_; }

	HtmlTag& tail(HtmlTag h)
	{ tail_ << h ; return tail_; }

	void sub_title(strlist collect, std::string text);
	
	void linkbar(std::string);

	void make_inst_link(strlist::iterator , strlist::iterator );
	void make_expid_link(strlist::iterator 
		, strlist::iterator 
		, strlist::iterator 
		, strlist::iterator
	);
	void make_name_link(strlist::iterator , strlist::iterator );
	void addTable(strlist , strtable , std::string ="");

	friend std::ostream &operator<<(std::ostream& , HtmlPage );
private:

	static bool initdone_;
	static char fill_;

	static std::string init_;
	static HtmlTag head_;
	HtmlTag title_; 
	static HtmlTag style_;
	static HtmlTag meta_;
	static HtmlTag tail_;

	std::string thispage_;

	static HtmlTag XMMLogo();
};

#endif
