#ifndef PPSSUMMS_OBSUMMARY
#define PPSSUMMS_OBSUMMARY
/* 
 * Read the observation summary file and produce a FITs file via the Dal.
 * This is required by ppssumm
 */

#include <cstdio>
#include <cstdlib>
#include <string>
#include <fstream>
#include <map>
#include "SAttribute.h"
#include "SColumn.h"
#include "STable.h"
#include "sFunctions.h"
#include "Dal.h"
#include "Tracer.h"
#include "errstr.h"


class ObsSummary 
{


public:
ObsSummary(std::string , DataSet* ds );
	
private:
ifstream ifs_;
DataSet* ods_;	

/* 
 * This is a slight fudge giving void functions a return mechanism.
 * I've done it this way because I may need to return a value from
 * half way through a set of nested functions without wanting to break
 * the nest (ie. INSTRUMENT->EXPOSURE->CONFIGURATION).
 */
std::string rtn_string; 


/* We must store a number of future attributes until we have
 * created the tables to add them to. 
 * We do this via a multimap where the int key gives the table number for
 * a given attribute to be added to.  See writeAttributes().
 */
multimap < unsigned int , SAttribute > Atts_;
void addAtt( unsigned int i , SAttribute a)
{
	Atts_.insert(Atts_.upper_bound(i) , make_pair ( i , a ) );
}
void writeAttributes(); 

// Get next line from the summary file skipping whole line comments
std::string readline(bool emptylinessignificant = 1);

// Extract useful information from comments at the top of summary file.
void preread(); 

// Things to do after reading the sumary file
void postread(); 

// summary file sections
void OBSERVATION();
void FILES();
void CONFIGURATION();
void INSTRUMENT();
void EXPOSURE(STable& );
void INSTCONFIG(STable& ); 
void PROPOSAL();
void DATA_QUALITY();

};

#endif
