#include "OmSummary.h"

OmSummary::~OmSummary()
{
	/* Need this to stop destructor destroying input data sets with the object */
	fitsum_ = NULL;
	pidx_ = NULL;

}

OmSummary::OmSummary(DataSet* fitsum , DataSet* pidx ) 
: PpssummCommon( fitsum , pidx )  
{
	/* The main contents */

	multi_exposure_title();
	multi_exposure_products(); 

	exposures_processed();

	exposure_specific_title();
	exposure_specific_products();


}

void OmSummary::multi_exposure_title()
{
	multi_title.clear();
	multi_title.push_back(colName("FILENAME"));
	multi_title.push_back(colName("CONTENT"));
}

void OmSummary::multi_exposure_products()
{
	errstr << "Entering multi_exposure_products" << Sas::message(Msg::App, Msg::Sparse);
	multi_table.clear();

	if ( pidx_->hasBlock("OMOBS") ) {
	
		multi_table.insert(multi_table.begin() , 2 , strlist() );
		strlist tmpf , tmpc;

		Table* tbl = pidx_->table("OMOBS");

		if (tbl->hasColumn("NAME")) {

			Column* col = tbl->column("NAME");
			stringColumn(col , back_inserter(tmpf) );
			std::transform( tmpf.begin()
				, tmpf.end() 
				, tmpf.begin() 
				, toString_<std::string>() 
			);
		}
		
		if (tbl->hasColumn("CONTENT")) {
			Column* col = tbl->column("CONTENT");
			stringColumn(col , back_inserter(tmpc) );
			std::transform( tmpc.begin() 
				, tmpc.end() 
				, tmpc.begin() 
				, toString_<std::string>() 
			);
		}

		/* Remove what we don't want.
		 * Filter on content first
		 */
	
		for ( strlist::size_type offset=0 ; offset < tmpc.size() ; offset++ ) {

			if  ( tmpc[offset].find("OBSERVATION SUMMARY") != std::string::npos)
				continue;

			multi_table[0].push_back ( tmpf[offset] );
			multi_table[1].push_back ( tmpc[offset] );

		}

		/* Sort what is left */
		if ( multi_table[0].size() && multi_table[1].size() ) {
			
			vector< strlist::difference_type > dt;
			sort_order( multi_table[1].begin()
				,multi_table[1].end()
				,back_inserter(dt)
			);

			reorder( multi_table[0].begin()
				,multi_table[0].end()
				,dt.begin()
				,dt.end()
			);
		} else {
			multi_table.clear();
		}

		
	}
	
	errstr << "Leaving multi_exposure_products" << Sas::message(Msg::App, Msg::Sparse);
	
}

void OmSummary::exposures_processed()
{
	errstr << "Entering exposures_processed" << Sas::message(Msg::App, Msg::Sparse);
	/* 
	 * Okay, this is the OM nothing is ever straight forward.
	 * We can have exposures in OMSRC and OMSW with no record in OMEXP (no exposure specific products)
	 *
	 * We should already have all necessay exposure data in PpssummCommon::exposure_tables.
	 * It must be filtered by the exposures actually processed which we get from pidx_
	 * each PpssummCommon::exposure_table[] is already sorted how we want them.
	 *
	 * As usual, things are never simple.  Just because an instrument is active does not
	 * mean any exposures were processed.  If no exposures were processed we won't find
	 * the associated table in pidx_.
	 */

	/* Bad hack, copied from PpssummCommon */
        enum ExpColId { 
                INSTRUMENT =0 
                ,EXPID    
                ,SCHEDULED 
                ,MODE      
                ,DATA_MODE 
                ,FILTER   
                ,FILTER_WHEEL  // PN Uses column name FILTER_POTENTIOMETER
                ,DURATION
                ,ACTUAL_START
                ,ACTUAL_END  
                ,NUMCOL
        };
	/* Note, the FILTER_WHEEL column will be skipped as we don't need it */

	exp_table.clear();


	std::string instruments[] = { "OM" , "" };

	for (std::string* inst = instruments ; inst->size() ; inst++ ) {

		keyed_tables::const_iterator PI = processed_tables_.find( *inst );

		if ( PI != processed_tables_.end() ) {
			
			const strtable& idata = PI->second;

			if ( ! exp_table.size() ) exp_table.insert( exp_table.begin() , NUMCOL -1 , strlist() );
			
			for ( unsigned int i = 0 ; i < idata.size() ; i++ ) {

				if ( i == FILTER_WHEEL) continue ;
				unsigned int j =  (i > FILTER_WHEEL) ? i - 1 : i;

				exp_table[j].insert( exp_table[j].end() , idata[i].begin() , idata[i].end() );
			}
		}	
	}

	errstr << "Leaving exposures_processed" << Sas::message(Msg::App, Msg::Sparse);
	
}

void OmSummary::exposure_specific_title()
{

	specific_title.clear();
	specific_title.push_back(colName("FILENAME"));
	specific_title.push_back(colName("CONTENT"));
	specific_title.push_back(colName("VVFLAGS"));
	specific_title.push_back(colName("OSW_ID"));
	specific_title.push_back(colName("SRC_NUM"));

}

void OmSummary::exposure_specific_products()
{
	errstr << "Entering exposure_specific_products" << Sas::message(Msg::App, Msg::Sparse);
	/* That bad hack again */

        enum ExpColId { 
                INSTRUMENT =0 
                ,EXPID    
                ,SCHEDULED 
                ,MODE      
                ,DATA_MODE 
   //            //Eliminated since ON doesn't use it.
   //             ,FILTER   
   //            ,FILTER_WHEEL  
                ,DURATION
                ,ACTUAL_START
                ,ACTUAL_END  
                ,NUMCOL
        };

	enum SpecColId {
		NAME = 0
		, CONTENT
		, VVFLAGS
		, OSW_ID
		, SRC_NUM
		, NUMSPECCOL
	};

	std::string inst("OM");
	if ( isActive( inst ) && exp_table.size() ) {

		/* Extract extra information for each exposure */
		/* Collect data on a per processed exposure basis, sort it later */
		
		for (strlist::const_iterator expid = exp_table[EXPID].begin()
			; expid != exp_table[EXPID].end()
			; expid++
		) {

			std::string key= inst + *expid;
			strtable& idata = specific_table[key];

			/* pidx_ tables exposure spcific data comes from */			
			const std::string tables[] = { 
				  "OMEXP" 
				, "OMSW" 
				, "OMSRC" 
				, "" 
			};
		
			/* The normal extra columns we want and which column they go into */
			const std::string want[] = {  
				  "NAME" 
				, "CONTENT" 
				, "VVFLAGS" 
				, "" 
			};
			
			const unsigned int to[] = { NAME , CONTENT , VVFLAGS  };

			/*
			 * Special columns 
			 * OSW_ID and SRC_NUM columns are not in all tables and are handled separately 
			 */
		
			for(const std::string* table =tables ; table->size() ; table++ ) {
				
				if (! pidx_->hasBlock(*table) ) continue;

				Table* tbl = pidx_->table(*table);
		
				strlist foundexp;
				Column* col = tbl->column("EXP_ID");
				stringColumn( col , back_inserter(foundexp) );
				std::transform( foundexp.begin() , foundexp.end() , foundexp.begin() , toString_<std::string>() );

				/* Identify rows of extra data matching the current exposure id */
				vector<strlist::difference_type> orderI;
				strlist::const_iterator Ab =  foundexp.begin();
				strlist::const_iterator Ae =  foundexp.end();

				/* Get all lines matching the current exposure id */
				find_all( Ab 
					, Ae 
					, *expid 
					, back_inserter(orderI)
				);

				/* skip if we didn't find any */
				if ( ! orderI.size() ) continue;
	
				/* Now read the rows of relevant columns filling not found columns with blanks */
				strtable tmpdata;
				tmpdata.insert( tmpdata.begin(), NUMSPECCOL , strlist() );

				/* 
				 * Read these columns we know we should have from pidx_ retrieving only 
				 * those refering to the exposure we are after
				 */

				for (unsigned int i=0; i < 3 ; i++) { 

					col = tbl->column(want[i]);
					stringColumn( col 
						,  back_inserter(tmpdata[to[i]]) 
						, orderI.begin() 
						, orderI.end() 
					);
					std::transform( tmpdata[to[i]].begin() 
						, tmpdata[to[i]].end() 
						, tmpdata[to[i]].begin() 
						, toString_<std::string>() 
					);
				}

				/* OSW_ID is not necessarily present, pad the column if not. */
				if ( tbl->hasColumn("OSW_ID") ) {
					col = tbl->column("OSW_ID");
					stringColumn( col 
						,  back_inserter(tmpdata[OSW_ID]) 
						, orderI.begin() 
						, orderI.end() 
					);
					std::transform( tmpdata[OSW_ID].begin() 
						, tmpdata[OSW_ID].end() 
						, tmpdata[OSW_ID].begin() 
						, toString_<std::string>() 
					);
					
					std::transform( tmpdata[OSW_ID].begin() 
						, tmpdata[OSW_ID].end() 
						, tmpdata[OSW_ID].begin() 
						, OmOswRemoveFilter_() 
					);

				} else {
					tmpdata[OSW_ID].insert(tmpdata[OSW_ID].end() , orderI.size() , std::string(""));
				}


				/* SRC_NUM is not necessarily present, pad the column if not. */
				/* SRC_NUM is int16 */
				if ( tbl->hasColumn("SRC_NUM") ) {
					col = tbl->column("SRC_NUM");
					vector<int16> vi;
					int16Column( col 
						, back_inserter(vi) 
						, orderI.begin() 
						, orderI.end() 
					);

					std::transform( vi.begin() 
						, vi.end() 
						, back_inserter(tmpdata[SRC_NUM])
						, toString_<int16>()
					);
				} else {
					tmpdata[SRC_NUM].insert(tmpdata[SRC_NUM].end() , orderI.size() , std::string(""));
				}

				/* Accumulate data to idata */
			
				if (! idata.size() ) idata.insert( idata.begin() , NUMSPECCOL , strlist() );
				for(unsigned int i = 0
					; i < idata.size()
					; i++
				) idata[i].insert(idata[i].end() , tmpdata[i].begin() , tmpdata[i].end());
				
			}

			/* skip if we didn't find anything for this exposure */
			if (! idata.size()) continue; 

			PpssummStatic::anchors_.push_back(key);
			/* Sort columns on OSW_ID then SRC_NUM then CONTENT */

			/* OSW_ID sort */
			typedef pair< strlist::difference_type, strlist::difference_type > offset_range; 

			vector< offset_range > offsetranges;

			vector< strlist::difference_type > row_order;
	
			sort_order( idata[OSW_ID].begin()
				, idata[OSW_ID].end()
				, back_inserter(row_order)
			);
		
			equal_ranges(idata.begin() , idata.end() , back_inserter(offsetranges));

			/* sub-sort SRC_NUM and CONTENT */
			unsigned int scol[] = { SRC_NUM , CONTENT };
			for ( unsigned int* i = scol ; i != scol + 2 ; i++ ) {

				/* First reorder as per current row_order */
				reorder( idata[*i].begin()
					, idata[*i].end()
					, row_order.begin()
					, row_order.end()
				);

				/*
				 * For each equal range of previous sorts sub-sort this column
				 * and reorder the row_roder for that range accordingly
				 */
				vector< offset_range > newranges;
				for ( vector< offset_range >::const_iterator CJ = offsetranges.begin()
					; CJ != offsetranges.end()
					; CJ++
				){

					std::vector<strlist::difference_type> sub_order;
					
					sort_order( idata[*i].begin() + CJ->first 
						, idata[*i].begin() + CJ->second  
						, std::back_inserter(sub_order) 
					);

					reorder( row_order.begin() + CJ->first 
						, row_order.begin() + CJ->second 
						, sub_order.begin() 
						, sub_order.end() 
					);

					/* 
					 * Find equal_range's of this column within the equal_ranges defined by previous columns.
					 * Remeber, equal ranges in this column outside of these prior equal ranges 
					 * have been sorted according to a previous column and should not be re-sorted.
					 */
					
					equal_ranges( idata[*i].begin() + CJ->first ,  idata[*i].begin() + CJ->second , back_inserter(newranges) );
				}
				offsetranges = newranges;
			}

			/* re-order the rest of the columns */
			unsigned int ocol[] = { 0 , 2  };
			for ( unsigned int* i = ocol ; i != ocol + 2 ; i++ ) {

				reorder( idata[*i].begin()
					, idata[*i].end()
					, row_order.begin()
					, row_order.end()
				);
			}
		}
	}
		
	errstr << "Leaving exposure_specific_products" << Sas::message(Msg::App, Msg::Sparse);
	
}

HtmlPage OmSummary::print()
{
	HtmlPage page_("omxpage");

	std::string htitle("Observation " + PpssummCommon::obsid_ + " Pipeline Processing Summary");
	errstr << "Generating " << htitle << Sas::message(Msg::App, Msg::Verbose);

	const HtmlTag hr("hr");

	/* Top of the page */ 
	page_.title( htitle );

	page_.linkbar("omxpage");
	page_.body_ << hr;
	
	page_.sub_title(PpssummCommon::sub_title_ , PpssummCommon::obsid_ + " OM Processing Summary");
	page_.body_ << hr;

	/* middle of page */
	print_multi_exposure_products(page_);
	print_exposures_processed(page_);
	print_exposure_specific_products(page_);

	/* end of page */

	return page_;
}


void OmSummary::print_multi_exposure_products(HtmlPage& page)
{
	page.body_ << HtmlTag("h3" , "" , "OM Multi-Exposure Products");
	if ( multi_table.size() ) {
		
		page.addTable(multi_title , multi_table);

	} else {
		page.body_ << HtmlTag("tt","","No multi-exposure products were produced.");
	}
	page.body_  << HtmlTag("hr");


}

void OmSummary::print_exposures_processed(HtmlPage& page)
{
        enum ExpColId { 
                INSTRUMENT =0 
                ,EXPID    
                ,SCHEDULED 
                ,MODE      
                ,DATA_MODE 
                ,FILTER   
                ,FILTER_WHEEL  //Eliminated since OM doesn't use it.
                ,DURATION
                ,ACTUAL_START
                ,ACTUAL_END  
                ,NUMCOL
        };

	page.body_ << HtmlTag("h3","","Exposures processed by PPS");

	if (exp_table.size() ) {

		exp_title = PpssummCommon::exposure_title_;
		exp_title.erase(exp_title.begin() + static_cast<strlist::difference_type>(FILTER_WHEEL));

		page.addTable( exp_title , exp_table );

	} else {
		page.body_ << HtmlTag("tt","","No OM exposures processed by the PPS.");

	}
	page.body_  << HtmlTag("hr");

}

void OmSummary::print_exposure_specific_products(HtmlPage& page) 
{
	page.body_ << HtmlTag("h3","","Exposure Specific Products");

	if (exp_table.size() && specific_table.size() ) {

		/* Loop over exp_table so we get specific products in the same order */
		for (unsigned int i = 0 ; i < exp_table[0].size() ; i++ ) {

			std::string key =  exp_table[0][i] + exp_table[1][i];
			keyed_tables::iterator MI = specific_table.find( key );

			/* ignore exposures where there are no specific products */
			if ( MI ==  specific_table.end() ) continue;
	
			/* Copy the exposure information into a separate table for output */
			strtable omtmp(exp_table.size());
			for (unsigned int j = 0 ; j < exp_table.size() ; j++) {
				omtmp[j] = strlist(1 , exp_table[j][i]);
			}
			
			page.addTable( exp_title , omtmp , key );
			page.addTable( specific_title , MI->second );
			page.body_ << HtmlTag("hr");
		}

	} else {
		page.body_ << HtmlTag("tt","","No exposure specific products were produced.");
		page.body_  << HtmlTag("hr");

	}

}
