#include "PpssummCommon.h"

/* define static members */
bool PpssummCommon::constructordone_ = false;

std::string PpssummCommon::obsid_;
std::string PpssummCommon::procdate_;
strlist PpssummCommon::sub_title_;
strlist PpssummCommon::instrument_title_;
strtable PpssummCommon::instrument_table_;
strlist PpssummCommon::exposure_title_;
keyed_tables PpssummCommon::exposure_tables_;
keyed_tables PpssummCommon::processed_tables_;

/* 
 * The destructor 
 * Set the DataSet* pointers to null.
 * We don't want to call the destructor of things which don't belong to us.
 */
PpssummCommon::~PpssummCommon()
{

	fitsum_ = NULL;
	pidx_ = NULL;
}


std::string PpssummCommon::dataDataMode(Table* tbl , unsigned int offset) const
{

	toString_<unsigned int> to_ui;

	strlist collect;

	/* 
	 * OM starts at DATA_MODE_0 
	 * EPIC and RGS at DATA_MODE_1
	 * RGS1 is missing DATA_MODE_7
	 * RGS2 is missing DATA_MODE_4
	 * These correspond to the knackered CCDs
	 */

	for(unsigned int i=0; i < 12 ; i++) {
		std::string colname = "DATA_MODE_" + to_ui(i);
		if ( tbl->hasColumn(colname) ) {
		
			Column* col = tbl->column(colname);
			stringColumn( col , back_inserter(collect) , offset , 1);
		}
	}

	std::string datamode("");
	if (collect.size()) {

		// DATA_MODE_n get translated into a single summary value
		

		std::transform( collect.begin() 
			, collect.end() 
			, collect.begin() 
			, toString_<std::string>()
		);

		strlist::iterator cb = collect.begin();
		while ( cb != collect.end() )
		{
			std::transform( cb->begin() 
				, cb->end() 
				, cb->begin() 
				, toUpper 
			); 
			cb++;
		}
		// The find("FAST") only applies to OM

		strlist::iterator fast = std::find( collect.begin() 
			, collect.end() 
			, std::string("FAST") 
		);
		
		strlist::iterator imaging = std::find( collect.begin() 
			, collect.end() 
			, std::string("IMAGING") 
		);
		
		strlist::iterator other = std::find_if( collect.begin() 
			, collect.end() 
			, notBlank_() 
		);

		if ( fast != collect.end() 
			&& imaging != collect.end() 
		) {
			datamode = "Imaging & Fast";

		} else if ( imaging != collect.end() ) {

			datamode = "Imaging";

		} else if ( fast != collect.end() ) {
			
			datamode = "Fast";

		} else if ( other  != collect.end() ) {


			datamode = *other;

			// Capitalize first letter, lowercase for others
			std::transform( datamode.begin()+1 
				, datamode.end() 
				, datamode.begin() 
				, toLower 
			);
			
		} else {
			datamode="";
		}
	}
	return datamode;

}

void PpssummCommon::dataSimpleValues()
{
	strlist collect;
	Table* tbl;
	tbl = fitsum_->table("OBSERVAT");
	if ( tbl->hasColumn("OBS_ID") ) {
		Column* col = tbl->column("OBS_ID");
		stringColumn ( col , back_inserter(collect) );
		std::transform( collect.begin() , collect.end() , collect.begin() , toString_<std::string>() );
		PpssummCommon::obsid_ = collect[0];
	} else {
		PpssummCommon::obsid_ = "Not Available";
	}

	tbl = pidx_->table(0);
	if (tbl->hasAttribute("PROCDATE") ) {
		PpssummCommon::procdate_ = tbl->attribute("PROCDATE")->asString();
	} else {
		PpssummCommon::procdate_ = "Not Available";

	}

}

void PpssummCommon::dataSubTitle()
{
	/* Extraxt info fro page sub titles from summary file */	
	std::string tname("OBSERVAT");
	if ( fitsum_->hasBlock(tname) ) {

		Table* tbl = fitsum_->table(tname);
		
		const std::string want[] = {
			 "OBS_ID"
			,"REVOLUTION"
			,"OBS_START"
			,"OBS_STOP"
			,""
		};
		const std::string* w = want;

		while ( w->length() ) {

			if ( tbl->hasColumn(*w) ) {
				Column* col = tbl->column(*w);
				stringColumn(col , back_inserter(PpssummCommon::sub_title_));

			} else {
				errstr << "Unable to find column "
					<< *w << " in table OBSERVAT of summary file."
					<< Sas::fatal("ppssumm:BADSUMMARYFILE");
					
			}
			w++;
		}
	} else {
		errstr << "Unable to find OBSERVAT table in sumary file" << Sas::fatal("sub_title:BADSUMMARYFILE");
	}
		
	tname = "PROPOSAL";
	if ( fitsum_->hasBlock(tname) ) {

		Table* tbl = fitsum_->table(tname);
		
		const std::string want[] = {
			 "TARGET"
			,"GONAME"
			,""
		};
		const std::string* w = want;

		while ( w->length() ) {
			if ( tbl->hasColumn(*w) ) {
				Column* col = tbl->column(*w);
				stringColumn(col , back_inserter(PpssummCommon::sub_title_));
			} else {
				errstr << "Unable to find column "
					<< *w << " in table PROPOSAL of summary file."
					<< Sas::fatal("ppssumm:BADSUMMARYFILE");
					
			}
			w++;
		}
	} else {
		errstr << "Unable to find PROPOSAL table in sumary file" << Sas::fatal("ppssumm:sub_title:BADSUMMARYFILE");
	}

}

void PpssummCommon::dataInstrumentTitles()
{
	PpssummCommon::instrument_title_.clear();
	PpssummCommon::instrument_title_.push_back(altName("INSTRUMENT"));
	PpssummCommon::instrument_title_.push_back(colName("ACTIVE"));
	PpssummCommon::instrument_title_.push_back(colName("NREXP")); 
	PpssummCommon::instrument_title_.push_back(colName("PRIORITY"));

}

void PpssummCommon::dataInstrumentTable()
{

	enum InstColId {
		INSTRUMENT=0
		,ACTIVE
		,NREXP
		,PRIORITY
		,NUMCOL
	};

	PpssummCommon::instrument_table_.clear();
	activeInstrument_.clear();

	/* instrument = 0 , active = 1 , nrexp = 2 , priority = 3 */

	PpssummCommon::instrument_table_.insert( PpssummCommon::instrument_table_.begin() 
		, NUMCOL 
		, strlist() 
	); 

	for (strlist::const_iterator inst = PpssummCommon::allInstrument_.begin() 
		; inst != PpssummCommon::allInstrument_.end() 
		; inst++ 
	) {

		PpssummCommon::instrument_table_[INSTRUMENT].push_back(instName(*inst));
		
	
		strlist collect;
		vector<Attribute*> instAtt;

		/* The ACTIVE flag and NREXP are recorded as attributes of the instrument tables */
		const std::string kw[] = {"ACTIVE" , "NREXP"};
		if ( fitsum_->hasBlock(*inst) ) {

			Table* tbl = fitsum_->table(*inst);
			
			getAttribute(tbl, kw , kw+2 , back_inserter(instAtt) , 1);
			PpssummCommon::instrument_table_[ACTIVE].push_back( instAtt[0]->asString() );
			PpssummCommon::instrument_table_[NREXP].push_back( instAtt[1]->asString() );

			std::string test = instAtt[0]->asString();
			if ( test[0] == 'Y'  || test[0] == 'T') activeInstrument_.push_back(*inst);
		}


		/* The Instrument priority is recorded as a column ii_PRIORITY of the PROPOSAL table */
		const std::string tname = "PROPOSAL";
		if (fitsum_->hasBlock(tname) ) {
	
			Table* tbl = fitsum_->table(tname);
			std::string cn(*inst + "_PRIORITY");
			if ( tbl->hasColumn( cn ) ) {
				Column* col = tbl->column(cn);
				stringColumn(col , back_inserter(PpssummCommon::instrument_table_[PRIORITY]));
			} else {
				errstr << "Unable to find column "
					<< cn << " in table OBSERVAT of summary file."
					<< Sas::fatal("ppssumm:BADSUMMARYFILE");
					
			}

		}
	}

	for(unsigned int i = 0; i < NUMCOL ; i++) std::transform( PpssummCommon::instrument_table_[i].begin() , PpssummCommon::instrument_table_[i].end() , PpssummCommon::instrument_table_[i].begin() , toString_<std::string>() );

	return;
	
}

void PpssummCommon::dataExposureTitles()
{
	PpssummCommon::exposure_title_.clear();
	PpssummCommon::exposure_title_.push_back(altName("INSTRUMENT"));
	PpssummCommon::exposure_title_.push_back(altName("EXPID"));
	PpssummCommon::exposure_title_.push_back(altName("SCHEDULED"));
	PpssummCommon::exposure_title_.push_back(colName("MODE"));
	PpssummCommon::exposure_title_.push_back(colName("DATA_MODE"));
	PpssummCommon::exposure_title_.push_back(colName("FILTER"));
	PpssummCommon::exposure_title_.push_back(altName("FILTER_WHEEL"));
	PpssummCommon::exposure_title_.push_back(colName("DURATION"));
	PpssummCommon::exposure_title_.push_back(colName("ACTUAL_START"));
	PpssummCommon::exposure_title_.push_back(colName("ACTUAL_END"));
}

void PpssummCommon::dataExposureTables()
{
	/*
	 * Desired columns are 
	 * Inst , ExpID  , Sched , Mode , Data Mode , Filter , Position , Actual Start , Actual Stop
	 *
	 * All are available from the instrument tables (M1 ,M2 ...)
	 * The Data Mode column is a condesation of all DATA_MODE_n columns
	 *
	 * The Duration column is calculated.
	 *
	 * Loop over each instrument skipping those which are missing or have empty
	 * tables.  Add a table of extracted data as exposure_tables['instrument']
	 *
	 * Warning, table will be empty if the instrument was not active so check and skip as required.
	 */

	enum ExpColId { 
		INSTRUMENT =0 
		,EXPID    
		,SCHEDULED 
		,MODE      
		,DATA_MODE 
		,FILTER   
		,FILTER_WHEEL  // PN Uses column name FILTER_POTENTIOMETER
		,DURATION
		,ACTUAL_START
		,ACTUAL_END  
		,NUMCOL
	};

	PpssummCommon::exposure_tables_.clear();
	
	for (strlist::const_iterator inst = allInstrument_.begin() ; inst != allInstrument_.end() ; inst++ ) {
		
		if ( ! fitsum_->hasBlock(*inst) ) continue;

		Table* tbl = fitsum_->table(*inst);

		if( ! tbl->columns() ) continue;
		if( ! tbl->rows() ) continue;

		/* 
		 * collect instrument specific data in temporary storage, process then copy
		 * into final storage
		 */
		keyed_tables::iterator MI;
		MI = exposure_tables_.insert( exposure_tables_.end() 
			, make_pair( *inst 
				, strtable(NUMCOL , strlist() ) 
			)
		);

		strtable& idata = MI->second;
		Column* col;
		strlist tmp;
	
		/* 
		 * Start with ACTUAL_START as this is the principal sort column
		 * We use the sort order to read other columns in the correct row order 
		 */
		
		col = tbl->column("ACTUAL_START");
		stringColumn ( col , back_inserter(idata[ACTUAL_START]) );
		std::transform( idata[ACTUAL_START].begin() , idata[ACTUAL_START].end() , idata[ACTUAL_START].begin() , toString_<std::string>() );

		vector<unsigned int> order;  
		sort_order( idata[ACTUAL_START].begin() , idata[ACTUAL_START].end() , std::back_inserter(order) ); 
		std::transform( idata[ACTUAL_START].begin() , idata[ACTUAL_START].end() , idata[ACTUAL_START].begin() , truncTime );

		/* We need as many copies of the instrument name as we have elements in idata[ACTUAL_START] */
		idata[INSTRUMENT].insert( idata[INSTRUMENT].begin() , order.size() , instName(*inst) );

		/* Other easy columns , toString_ removes trailing \0 */

		col = tbl->column("EXPID");
		stringColumn ( col , back_inserter(idata[EXPID]) , order.begin() , order.end() );
		std::transform( idata[EXPID].begin() , idata[EXPID].end() , idata[EXPID].begin() , toString_<std::string>() );
		
		col = tbl->column("SCHEDULED");
		stringColumn ( col , back_inserter(idata[SCHEDULED]) , order.begin() , order.end() );
		std::transform( idata[SCHEDULED].begin() , idata[SCHEDULED].end() , idata[SCHEDULED].begin() , toString_<std::string>() );
	
		/* 
		 * There can be two exposures of the same expid, distinguished by one being scheduled
		 * the other not.  We prepend the standard S/U to the EXPID here because it is more usefull
		 * with it
		 */

		for (unsigned int i = 0 ; i < idata[SCHEDULED].size() ; i++ ) {
			if ( idata[SCHEDULED][i] == "Y" ) 
			{
				idata[EXPID][i].insert(0 , "S");
			
			} else if ( idata[SCHEDULED][i] == "N" ) {
				
				idata[EXPID][i].insert(0 , "U");
			} else {
			
				errstr << "Do not understand Scheduled flag" << Sas::error("ppssumm::BadScheduledFlag");
			}
		}

		if ( tbl->hasColumn("MODE") )
		{
			col = tbl->column("MODE");
			stringColumn ( col , back_inserter(idata[MODE]) , order.begin() , order.end() );
			std::transform( idata[MODE].begin() , idata[MODE].end() , idata[MODE].begin() , toString_<std::string>() );
		} else {
			idata[MODE].insert( idata[MODE].begin() , order.size() , "" );
		}

		if ( tbl->hasColumn("ACTUAL_END") )
		{
			col = tbl->column("ACTUAL_END");
			stringColumn ( col , back_inserter(idata[ACTUAL_END]) , order.begin() , order.end() );
			std::transform( idata[ACTUAL_END].begin() , idata[ACTUAL_END].end() , idata[ACTUAL_END].begin() , toString_<std::string>() );
			std::transform( idata[ACTUAL_END].begin() , idata[ACTUAL_END].end() , idata[ACTUAL_END].begin() , truncTime );
		} else {
			idata[ACTUAL_END].insert( idata[ACTUAL_END].begin() , order.size() , "" );
		}
		


		/* Now the fussier columns */

		/* DURATION */

		if (tbl->hasColumn("ACTUAL_START") && tbl->hasColumn("ACTUAL_END") )
		{
			toString_<double> diff;
			for(strlist::size_type i = 0 ; i< idata[ACTUAL_START].size() ; i++ ) {
				STime stop(idata[ACTUAL_END][i]);
				STime start(idata[ACTUAL_START][i]);
				double d = stop - start;
				idata[DURATION].push_back( diff(d) ) ;
			}
		} else {

			idata[DURATION].insert( idata[DURATION].begin() , order.size() , "" );
			
		}
		
		/* DATA_MODE 
		 * There are N DATA_MODE_n columns and we need to read each per exposure and 
		 * concentrate the results into a single value.  The only instrument which
		 * is meant to be able to have differint non-blank DATA_MODE_n values is the OM
		 *
		 * Concentration rules are:
		 *	For OM if we find Imaging and Fast then  = Imaging & Fast
		 *	otherwise if "Imaging" or "Fast" are among the values then take this.
		 *	otherwise take the first non-blank mode if there is one.
		 * 
		 */
		for( vector<unsigned int>::const_iterator VI = order.begin()
			; VI != order.end()
			; VI++
		) { 
			idata[DATA_MODE].push_back( dataDataMode( tbl , *VI ) ); 
		}

		/* 
		 * FILTER  POSITION - Only required by EPIC and OM 
		 * NB. I've separated the FILTER & FILTER_WHEEL ifs because
		 * it makes the code easier to read
		 */
		
		if ( *inst == "R1" || *inst == "R2" || ! tbl->hasColumn("FILTER") ) {

			idata[FILTER].insert( idata[FILTER].begin() , order.size() , "" );

		} else {
		
			col = tbl->column("FILTER");
			stringColumn ( col , back_inserter(idata[FILTER]) , order.begin() , order.end() );
			std::transform( idata[FILTER].begin() , idata[FILTER].end() , idata[FILTER].begin() , toString_<std::string>() );
		}


		/* FILTER WHEEL/POTENTIOMETER - Only required by EPIC */


		if ( *inst == "PN" && tbl->hasColumn("FILTER_POTENTIOMETER")  ) {

			col = tbl->column("FILTER_POTENTIOMETER");
			stringColumn ( col , back_inserter(idata[FILTER_WHEEL]) , order.begin() , order.end() );
			std::transform( idata[FILTER_WHEEL].begin() , idata[FILTER_WHEEL].end() , idata[FILTER_WHEEL].begin() , toString_<std::string>() );

		} else if ( ( *inst == "M1" ||  *inst == "M2") && tbl->hasColumn("FILTER_WHEEL") ) {
			
			col = tbl->column("FILTER_WHEEL");
			stringColumn ( col , back_inserter(idata[FILTER_WHEEL]) , order.begin() , order.end() );
			std::transform( idata[FILTER_WHEEL].begin() , idata[FILTER_WHEEL].end() , idata[FILTER_WHEEL].begin() , toString_<std::string>() );

		} else {

			idata[FILTER_WHEEL].insert( idata[FILTER_WHEEL].begin() , order.size() , "" );

		}
	}

	return;
	
}

void PpssummCommon::dataProcessedTables()
{
	/* 
	 * We should already have all exposure data in PpssummCommon::exposure_tables.
	 * It must be filtered by the exposures actually processed which we get from pidx_
	 * each PpssummCommon::exposure_table[] is already sorted how we want them.
	 *
	 * As usual, things are ever simple.  Just because an instrument is active does not
	 * mean any exposures were processed.  If no exposures were processed we won't find
	 * the associated table in pidx_.
	 */

	/* Bad hack, copied from above, will be fixed when I 'class'ify things a little better */
        enum ExpColId { 
                INSTRUMENT =0 
                ,EXPID    
                ,SCHEDULED 
                ,MODE      
                ,DATA_MODE 
                ,FILTER   
                ,FILTER_WHEEL  // PN Uses column name FILTER_POTENTIOMETER
                ,DURATION
                ,ACTUAL_START
                ,ACTUAL_END  
                ,NUMCOL
        };

	processed_tables_.clear();

	for (strlist::const_iterator inst = activeInstrument_.begin()
		; inst != activeInstrument_.end()
		; inst++
	) {

		/* 
		 * Find the exposure_table for this insturment first 
		 * Just because an insturment is active doesn't mean there are exposures for it
		 */

		keyed_tables::const_iterator exptable = exposure_tables_.find( *inst );
		if (exptable == exposure_tables_.end() ) continue;

		/* translate from instrument to appropriate pidx_ table(s)*/
		strlist tablenames;
		if ( *inst == "M1" || *inst == "M2" || *inst == "PN" ) {
			
			tablenames.push_back("EPICEXP");

		} else if ( *inst == "R1" || *inst == "R2" ) {

			tablenames.push_back( "RGSEXP" );
			tablenames.push_back( "RGSSRC" );

		} else if ( *inst == "OM" ) {

			tablenames.push_back( "OMEXP" );
			tablenames.push_back( "OMSW" );
			tablenames.push_back( "OMSRC" );

		}
		
		/* Accumulate list of all expid refered to in the relevant 
		 * pidx_ tables.  It is necessary to filter on instrument
		 * as the exposure id is not unique across instruments.
		 * 
		 */
		strlist expid;
		vector< strlist::difference_type > rows;
		for (strlist::const_iterator tablename = tablenames.begin()
			; tablename != tablenames.end()
			; tablename ++
		) {

			rows.clear();
			if (  ! pidx_->hasBlock(*tablename) ) continue; 
	
			Table* tbl = pidx_->table(*tablename);
			Column* col = tbl->column("INSTRUMENT");
			strlist allinst;
			stringColumn(col , back_inserter(allinst));
			std::transform( allinst.begin() , allinst.end() , allinst.begin() , toString_<std::string>() );
			const std::string longinst[] = { instName(*inst) , "" };
			strlist::const_iterator Ab = allinst.begin();
			strlist::const_iterator Ae = allinst.end();
			find_all( Ab , Ae , *longinst , back_inserter(rows));
	
			col = tbl->column("EXP_ID");
			stringColumn(col , back_inserter(expid) , rows.begin() , rows.end() );
		}

		std::transform( expid.begin() , expid.end() , expid.begin() , toString_<std::string>() );
		
		std::sort( expid.begin() , expid.end() );
		std::unique( expid.begin() , expid.end() );

		/* Get all exposure_table rows refering to processed exposures for this instrument */
		
		rows.clear();
		const strtable& idata = exptable->second;

		strlist::const_iterator Eb = expid.begin();
		strlist::const_iterator Ee = expid.end();

		find_all( idata[EXPID].begin()
			, idata[EXPID].end()
			, Eb 
			, Ee 
			, back_inserter(rows)
		);

		/* skip if we didn't find any */
		if ( ! rows.size() ) continue; 

		/*
		 * We found some.  Add a table for this instrument and
		 * copy them across from the exposures_table 
		 */

		keyed_tables::iterator proctable;
		proctable = processed_tables_.insert(processed_tables_.begin() 
			, make_pair( *inst , strtable( idata.size() , strlist() ) )
		); 
		strtable& exp_table = proctable->second;

		/* Copy exposure_table data to processed_tables */
		for (unsigned int i = 0; i < idata.size() ; i++) {

			for (vector<strlist::difference_type>::const_iterator row = rows.begin()
				; row != rows.end()
				; row++
			) {

					exp_table[i].push_back( idata[i][*row] );
			}
		}
			
	}

	return;

}
