#ifndef PPSSUMM_PPSSUMMCOMMON
#define PPSSUMM_PPSSUMMCOMMON

#include <cctype>
#include <algorithm>
#include <functional>
#include <iterator>
#include "ppssumm_typedefs.h"
#include "PpssummStatic.h"
#include "FitsTable.h"
#include "FitsColumn.h"
#include "ExtendFitsColumn.h"
#include "string_algo.h"
#include "STime.h"
#include "Dal.h"
#include "Tracer.h"
#include "errstr.h"

class PpssummCommon
:public PpssummStatic
{
public:
	/* 
	 * This class wraps common data and functions to construct it
	 * This data depends on the input files.
	 */

	/*
	 * fits summary file and product index. Needed by all pages.
	 */
	DataSet* fitsum_;
	DataSet* pidx_;

	PpssummCommon( DataSet* fitsum , DataSet* pidx ) 
	: PpssummStatic()
	{ 
		fitsum_ = fitsum;
		pidx_ = pidx;
		if (! constructordone_ ) {
			dataSimpleValues();
			dataSubTitle();
			dataInstrumentTitles();
			dataInstrumentTable();
			dataExposureTitles();
			dataExposureTables();
			dataProcessedTables();
			constructordone_ = true;
		}
	
	} 
	~PpssummCommon(); 

	/* The observation id */
	static std::string obsid_;
	static std::string procdate_;
		
	/* Data used to construct the htmlpage sub title */
	static strlist sub_title_;

	/* Details about each instrument */	
	static strlist instrument_title_;
	static strtable instrument_table_;

	/* Details from all exposures available */
	static strlist exposure_title_;
	static keyed_tables exposure_tables_;

	/* Details about exposures actually processed */
	static keyed_tables processed_tables_;

	/* Extra usefull functions */
	
	std::string dataDataMode(Table* tbl , unsigned int offset) const; 

	/* exposures avaialbe which were actually processed */
	
private: 

	/* These flags control initialization and the constructor */
	static bool constructordone_;
	void dataSimpleValues();  
	void dataSubTitle();  
	void dataInstrumentTitles();
	void dataInstrumentTable();
	void dataExposureTitles();
	void dataExposureTables();
	void dataProcessedTables();

};

#endif
