#ifndef PPSSUMM_PPSSUMM_STATIC
#define PPSSUMM_PPSSUMM_STATIC

#include <algorithm>
#include "ppssumm_typedefs.h"
#include "string_algo.h"
#include "Task.h"
#include "param.h"
#include "Tracer.h"
#include "errstr.h"

class PpssummStatic
{
public:
	/* 
	 * This class wraps data which span the different summary pages.
	 *
	 */


	PpssummStatic()
	{ 
		if ( ! initdone_ ) PpssummStatic::init();
	} 


	static void init() 
	{
		initVersion();
		initPageNames();
		initInstrument();
		initEnergyband();
		initColname();
		initAltname();
		initdone_ = true;

	}

	/* Here are some functions for accessing the data 
	 * contained in this class
	 */

	std::string version() const
	{ return PpssummStatic::ppssummVersion_; }

	std::string band( std::string b  ) const 
	{
		if ( ! b.size() ) return std::string("");

		std::transform( b.begin() , b.end() , b.begin() , toUpper );
	
		ssmap::const_iterator found;
		std::string rtn(b);
		found = PpssummStatic::energyband_.find(b);

		if (found != energyband_.end()) rtn = found->second;
		else rtn = std::string(""); 
			

		return rtn;
	}
	
	std::string altName( std::string c  ) const 
	{
		if ( ! c.size() ) return std::string("");
	
		std::transform( c.begin() , c.end() , c.begin() , toUpper );

		ssmap::const_iterator found;
		std::string rtn(c);
		found = PpssummStatic::altname_.find(c);

		if (found != altname_.end()) rtn = found->second;
		else rtn = colName(c); 
			

		return rtn;
	}

	std::string colName( std::string c ) const
	{
		if ( ! c.size() ) return std::string("");
		
		std::transform( c.begin() , c.end() , c.begin() , toUpper );
	
		ssmap::const_iterator found;
		std::string rtn(c);
		found = PpssummStatic::colname_.find(c);
		
		if (found != PpssummStatic::colname_.end()) rtn = found->second;
		else rtn = reformat(c);

		return rtn;
	}

	std::string instName( std::string c ) const 
	{
		if ( ! c.size() ) return std::string("");
		
		std::transform( c.begin() , c.end() , c.begin() , toUpper );

		ssmap::const_iterator found;
		std::string rtn(c);
		found = PpssummStatic::instname_.find(c);
		
		if (found != PpssummStatic::instname_.end()) rtn = found->second;
		else rtn = reformat(c);

		return rtn;
	}

	bool isActive(std::string s) const
	{
		return std::find( PpssummStatic::activeInstrument_.begin() , PpssummStatic::activeInstrument_.end() , s) != PpssummStatic::activeInstrument_.end();
	
	}

	bool hasAnchor(std::string s) const
	{

		return std::find( PpssummStatic::anchors_.begin() , PpssummStatic::anchors_.end() , s) != PpssummStatic::anchors_.end();

	}

	std::string whichPage(std::string p) 
	{
		if ( ! p.size() ) return std::string("");

		std::transform( p.begin() , p.end() , p.begin() , toUpper );

		std::string rtn("");
		ssmap::const_iterator found = PpssummStatic::pageNames_.find(p);

		if (found != PpssummStatic::pageNames_.end()) rtn = found->second;
		else rtn = std::string(""); 

		return rtn;
	}


	/* 
	 * Used to store translations form input names to output text
	 * and comman data used by the above functions 
	 */

	/*
	 * These are public because it is useful to be able to access them
	 * directly.  
	 */
	static strlist allInstrument_;
	static strlist activeInstrument_;
	static strlist anchors_;
	static ssmap pageNames_;

private: 
	/* Task version */
	static std::string ppssummVersion_;

	/* This flags control initialization and the constructor */
	static bool initdone_;

	static ssmap instname_;
	static ssmap energyband_;
	static ssmap colname_;
	static ssmap altname_;

	/* Class init needing done once */
	static void initVersion();
	static void initPageNames();
	static void initInstrument();
	static void initEnergyband();
	static void initColname();
	static void initAltname();

	// This is used sufficiently often it is worthwile putting it here

	std::string reformat(const std::string s) const
	{
		/* 
		 * A fall back function for undefiend name-> column title translations
		 * simply add a '+ to make it obvious.
		 */
		std::string rtn(s);
		return rtn+"+";
		
	}


};

#endif
