#include "ProductSummary.h"


ProductSummary::~ProductSummary()
{
	/* Need this to stop destructor destroying input data sets with the object */
	fitsum_ = NULL;
	pidx_ = NULL;

}

ProductSummary::ProductSummary(DataSet* fitsum , DataSet* pidx ) 
: PpssummCommon( fitsum , pidx )  
{
	/* The main contents */

	exposures_processed();
	
	generic_title();
	generic_products(); 

}

void ProductSummary::exposures_processed()
{
	/* 
	 * We should already have all necessay data in PpssummCommon::processed_tables_.
	 */

	/* Bad hack, copied from PpssummCommon */
        enum ExpColId { 
                INSTRUMENT =0 
                ,EXPID    
                ,SCHEDULED 
                ,MODE      
                ,DATA_MODE 
                ,FILTER   
                ,FILTER_WHEEL  // PN Uses column name FILTER_POTENTIOMETER
                ,DURATION
                ,ACTUAL_START
                ,ACTUAL_END  
                ,NUMCOL
        };

	exp_table.clear();

	for (strlist::const_iterator inst = activeInstrument_.begin()
		; inst != activeInstrument_.end()
		; inst ++
	) {

		keyed_tables::const_iterator PI = processed_tables_.find( *inst );

		if ( PI != processed_tables_.end() ) {
			
			const strtable& idata = PI->second;
	
			if ( ! exp_table.size() ) exp_table.insert( exp_table.begin() , NUMCOL , strlist() );
			
			for ( unsigned int i = 0 ; i < idata.size() ; i++ ) {

				exp_table[i].insert( exp_table[i].end() , idata[i].begin() , idata[i].end() );
			}
		}	
	}

}

void ProductSummary::generic_title()
{
	gen_title.clear();
	gen_title.push_back(colName("FILENAME"));
	gen_title.push_back(colName("CONTENT"));
}

void ProductSummary::generic_products()
{

	gen_table.clear();
	gen_table.insert(gen_table.begin() , 2 , strlist() );
	strlist tmpf , tmpc;

	if ( pidx_->hasBlock("PPSOBS") ) {
	
		Table* tbl = pidx_->table("PPSOBS");
		strlist filenames;
		strlist contents;

		if (tbl->hasColumn("NAME")) {


			Column* col = tbl->column("NAME");
			stringColumn(col , back_inserter(tmpf) );
			std::transform( tmpf.begin()
				, tmpf.end() 
				, tmpf.begin() 
				, toString_<std::string>() 
			);

			
			if (tbl->hasColumn("CONTENT")) {
				Column* col = tbl->column("CONTENT");
				stringColumn(col , back_inserter(tmpc) );
				std::transform( tmpc.begin() 
					, tmpc.end() 
					, tmpc.begin() 
					, toString_<std::string>() 
				);
			}

			/* Remove what we don't want.
			 * Filter on content first
			 */
		
			for ( strlist::size_type offset=0 ; offset < tmpc.size() ; offset++ ) {


				if ( tmpc[offset].find("LOGO") != std::string::npos ) continue;

				if  (tmpc[offset].find("PPS") != std::string::npos 
				&& tmpc[offset].find("INDEX") == std::string::npos
				) continue;
				
				gen_table[0].push_back ( tmpf[offset] );
				gen_table[1].push_back ( tmpc[offset] );
				

			}
			
		}
	}
}

HtmlPage ProductSummary::print()
{
	HtmlPage page_("ppspage");

	std::string htitle("Observation " + PpssummCommon::obsid_ + " Pipeline Processing Summary");
	errstr << "Generating " << htitle << Sas::message(Msg::App, Msg::Verbose);
	const HtmlTag hr("hr");

	/* Top of the page */ 
	page_.title( htitle );

	page_.linkbar("ppspage");
	page_.body_ << hr ;
	
	page_.sub_title(PpssummCommon::sub_title_ , PpssummCommon::obsid_ + " PPS Processing Summary");
	page_.body_ << hr ;

	/* Middle of page */
	print_exposures_processed(page_);
	print_generic_products(page_);
	
	/* end of page */

	return page_;	

}
void ProductSummary::print_exposures_processed(HtmlPage& page)
{
	page.body_ << HtmlTag("h3","","Exposures processed by PPS");

	if (exp_table.size() ) {
		page.addTable( exposure_title_ , exp_table );
	} else {
		page.body_ << HtmlTag("tt","","No Exposures were processed.");

	}
	page.body_ << HtmlTag("hr");

}

void ProductSummary::print_generic_products(HtmlPage& page)
{
	page.body_ << HtmlTag("h3","","Generic PPS Products");

	if ( gen_table.size() ) {
		page.addTable(gen_title , gen_table);
	} else {
		page.body_ << HtmlTag("tt","","No Generic PPS Products");
	}
	page.body_ << HtmlTag("hr");


}

