#include "RgsSummary.h"

RgsSummary::~RgsSummary()
{
	/* Need this to stop destructor destroying input data sets with the object */
	fitsum_ = NULL;
	pidx_ = NULL;

}

RgsSummary::RgsSummary(DataSet* fitsum , DataSet* pidx ) 
: PpssummCommon( fitsum , pidx )  
{
	/* The main contents */

	multi_exposure_title();
	multi_exposure_products(); 
	
	exposures_processed();

	exposure_specific_title();
	exposure_specific_products();

}

void RgsSummary::multi_exposure_title()
{
	multi_title.clear();
	multi_title.push_back(colName("FILENAME"));
	multi_title.push_back(colName("CONTENT"));
}

void RgsSummary::multi_exposure_products()
{
	errstr << "Entering multi_exposure_products" << Sas::message(Msg::App, Msg::Sparse);

	multi_table.clear();

	if ( pidx_->hasBlock("RGSOBS") ) {
	
		multi_table.insert(multi_table.begin() , 2 , strlist() );
		strlist tmpf , tmpc;

		Table* tbl = pidx_->table("RGSOBS");

		if (tbl->hasColumn("NAME")) {

			Column* col = tbl->column("NAME");
			stringColumn(col , back_inserter(tmpf) );
			std::transform( tmpf.begin()
				, tmpf.end() 
				, tmpf.begin() 
				, toString_<std::string>() 
			);
		}
		
		if (tbl->hasColumn("CONTENT")) {
			Column* col = tbl->column("CONTENT");
			stringColumn(col , back_inserter(tmpc) );
			std::transform( tmpc.begin() 
				, tmpc.end() 
				, tmpc.begin() 
				, toString_<std::string>() 
			);
		}

		/* Remove what we don't want.
		 * Filter on content first
		 */
		for ( strlist::size_type offset=0 ; offset < tmpc.size() ; offset++ ) {

			if  ( tmpc[offset].find("OBSERVATION SUMMARY") != std::string::npos)
				continue;

			multi_table[0].push_back ( tmpf[offset] );
			multi_table[1].push_back ( tmpc[offset] );

		}

		/* Sort what is left */
		if ( multi_table[0].size() && multi_table[1].size() ) {
			
			vector< strlist::difference_type > dt;
			sort_order( multi_table[1].begin()
				,multi_table[1].end()
				,back_inserter(dt)
			);

			reorder( multi_table[0].begin()
				,multi_table[0].end()
				,dt.begin()
				,dt.end()
			);
		} else {
			multi_table.clear();
		}

		
	}

}

void RgsSummary::exposures_processed()
{
	errstr << "Entering exposures_processed" << Sas::message(Msg::App, Msg::Sparse);
	/* 
	 * We should already have all necessay exposure data in PpssummCommon::processed_tables_.
	 *
	 * As usual, things are never simple.  Just because an instrument is active does not
	 * mean any exposures were processed.  If no exposures were processed we won't find
	 * the associated table in pidx_.
	 */

	/* Bad hack, copied from PpssummCommon */
	
        enum ExpColId { 
                INSTRUMENT =0 
                ,EXPID    
                ,SCHEDULED 
                ,MODE      
                ,DATA_MODE 
                ,FILTER   
                ,FILTER_WHEEL  // PN Uses column name FILTER_POTENTIOMETER
                ,DURATION
                ,ACTUAL_START
                ,ACTUAL_END  
                ,NUMCOL
        };

	exp_table.clear();
	
	std::string instruments[] = { "R1" , "R2" , "" };

	for (std::string* inst = instruments ; inst->size() ; inst++ ) {


		keyed_tables::const_iterator PI = processed_tables_.find( *inst );

		if ( PI != processed_tables_.end() ) {
			
			const strtable& idata = PI->second;

			if ( ! exp_table.size() ) exp_table.insert( exp_table.begin() , NUMCOL-2 , strlist() );
			
			for ( unsigned int i = 0 ; i < idata.size() ; i++ ) {

				/* FILTER and FILTER_WHEEL are irrelevant for RGS */
				if ( i == FILTER || i == FILTER_WHEEL ) continue;
				unsigned int j = (i > FILTER_WHEEL) ? i-2 : i;
				exp_table[j].insert( exp_table[j].end() , idata[i].begin() , idata[i].end() );
			}
		}	
	}

	errstr << "Leaving exposures_processed" << Sas::message(Msg::App, Msg::Sparse);
	
}

void RgsSummary::exposure_specific_title()
{

	specific_title.clear();
	specific_title.push_back(colName("FILENAME"));
	specific_title.push_back(colName("CONTENT"));
	specific_title.push_back(colName("VVFLAGS"));
	specific_title.push_back(colName("SRC_NUM"));
	specific_title.push_back(colName("RGSORDER"));

}

void RgsSummary::exposure_specific_products()
{
	typedef pair< strlist::difference_type, strlist::difference_type > offset_range; 

	errstr << "Entering exposure_specific_products" << Sas::message(Msg::App, Msg::Sparse);
	/* That bad hack again */

        enum ExpColId { 
                INSTRUMENT =0 
                ,EXPID    
                ,SCHEDULED 
                ,MODE      
                ,DATA_MODE 
                ,DURATION
                ,ACTUAL_START
                ,ACTUAL_END  
                ,NUMCOL
        };

	enum SpecColId {
		NAME = 0
		,CONTENT
		,VVFLAGS
		,SRC_NUM
		,RGSORDER
		,NUMSPECCOL
	};

	const std::string instruments[] = { 
		"RGS1" 
		, "RGS2" 
		, "" 
	};

	/* 
	 * pidx_ tables exposure spcific data comes from
	 * These tables are mixed RGS1 and RGS2 so we need to do the
	 * first selection based on intrument, then exposure id
	 */			
	const std::string tables[] = { 
		  "RGSEXP" 
		, "RGSSRC" 
		, "" 
	};

	/* The normal extra columns we want and which column they go into */
	const std::string want[] = {  
		  "NAME" 
		, "CONTENT" 
		, "VVFLAGS" 
		, "" 
	};
	
	const unsigned int to[] = { NAME , CONTENT , VVFLAGS  };

	/*
	 * Special columns 
	 * RGSORDER and SRC_NUM columns are not in all tables and are handled separately 
	 */


	for (const std::string* inst = instruments ; inst->size() ; inst++ ) {

		/* Is the instrument active ? */
		if (! isActive( PpssummCommon::instName(*inst) ) ) continue;

		/* Are there any processed exposures */
		keyed_tables::const_iterator PI = processed_tables_.find( PpssummCommon::instName(*inst) );
		if ( PI == processed_tables_.end() ) continue;
	
		for(const std::string* table =tables ; table->size() ; table++ ) {
			
			if (! pidx_->hasBlock(*table) ) continue;

			Table* tbl = pidx_->table(*table);
			Column* col;
		
			/* First select on instrument */
			col = tbl->column("INSTRUMENT");

			strlist foundinst;
			stringColumn( col , back_inserter( foundinst ) );
			std::transform( foundinst.begin() 
				, foundinst.end() 
				, foundinst.begin() 
				, toString_<std::string>() 
			);
			

			vector<strlist::difference_type> inst_rows;
			strlist::const_iterator Ib =  foundinst.begin();
			strlist::const_iterator Ie =  foundinst.end();


			/* Get all rows matching the current instrument */
			find_all( Ib 
				, Ie 
				, *inst 
				, back_inserter(inst_rows)
			);

				
			std::sort(inst_rows.begin() , inst_rows.end());
			std::unique(inst_rows.begin() , inst_rows.end() );

			/* skip if we didn't find in this table */
			if ( ! inst_rows.size() ) continue;

			/* We need all of the exposure ids */
			strlist foundexp;
			col = tbl->column("EXP_ID");
			stringColumn( col , back_inserter(foundexp) , inst_rows.begin() , inst_rows.end() );
			std::transform( foundexp.begin() , foundexp.end() , foundexp.begin() , toString_<std::string>() );

			std::vector<strlist::difference_type> exp_pos;
			sort_order( foundexp.begin() , foundexp.end() , back_inserter(exp_pos) );

			reorder(inst_rows.begin() , inst_rows.end() , exp_pos.begin() , exp_pos.end() );


                	vector< offset_range > offsetranges;

                	equal_ranges(foundexp.begin() , foundexp.end() , back_inserter(offsetranges));

			for ( vector< offset_range >::const_iterator offset = offsetranges.begin()
				; offset != offsetranges.end()
				; offset ++ 
			) {

				std::string expid = *(foundexp.begin() + offset->first);
				vector<strlist::difference_type> rows( inst_rows.begin() + offset->first
					, inst_rows.begin() + offset->second 
				);
				
						
				/* Now read the rows of relevant columns filling not found columns with blanks */
				std::string key= *inst + expid;
				strtable& idata = specific_table[key];
				if ( ! idata.size() ) idata.insert( idata.begin(), NUMSPECCOL , strlist() );

				/* 
				 * Read these columns we know we should have from pidx_ retrieving only 
				 * those refering to the exposure we are after
				 */

				for (unsigned int i=0; i < 3 ; i++) { 

					col = tbl->column(want[i]);
					stringColumn( col 
						,  back_inserter(idata[to[i]]) 
						, rows.begin() 
						, rows.end() 
					);
				}

				/* RGSORDER is not necessarily present, pad the column if not. */
				/* RGSORDER is int16 */
				if ( tbl->hasColumn("RGSORDER") ) {
					col = tbl->column("RGSORDER");
					vector<int16> vi;
					int16Column( col 
						,  back_inserter(vi) 
						, rows.begin() 
						, rows.end() 
					);

					std::transform( vi.begin() 
						, vi.end() 
						, back_inserter(idata[RGSORDER])
						, toString_<int16>()
					);

				} else {
					idata[RGSORDER].insert(idata[RGSORDER].end() , rows.size() , std::string(""));
				}


				/* SRC_NUM is not necessarily present, pad the column if not. */
				/* SRC_NUM is int16 */
				if ( tbl->hasColumn("SRC_NUM") ) {
					col = tbl->column("SRC_NUM");
					vector<int16> vi;
					int16Column( col 
						, back_inserter(vi) 
						, rows.begin() 
						, rows.end() 
					);

					std::transform( vi.begin() 
						, vi.end() 
						, back_inserter(idata[SRC_NUM])
						, toString_<int16>()
					);
				} else {
					idata[SRC_NUM].insert(idata[SRC_NUM].end() , rows.size() , std::string(""));
				}

				/* Clean up idata content */
				for( strtable::iterator CI = idata.begin()
					; CI != idata.end()
					; CI++
				) {
					std::transform( CI->begin() 
						, CI->end() 
						, CI->begin() 
						, toString_<std::string>() 
					);
				}
			}
		}

	}

	/* Now sort accumulated tables */
		
 	for (keyed_tables::iterator CI = specific_table.begin()
		; CI != specific_table.end()
		; CI++
	) {
		PpssummStatic::anchors_.push_back(CI->first);
                strtable& idata = CI->second;

		/* skip if we didn't find anything for this exposure */
		if (! idata.size()) continue; 

		/* Sort columns on SRC_NUM then RGSORDER then CONTENT */

		/* SRC_NUM sort */

		vector< offset_range > offsetranges;

		vector< strlist::difference_type > row_order;

		sort_order( idata[SRC_NUM].begin()
			, idata[SRC_NUM].end()
			, back_inserter(row_order)
		);
	
		equal_ranges(idata.begin() , idata.end() , back_inserter(offsetranges));

		/* sub-sort RGSORDER and CONTENT */
		unsigned int scol[] = { RGSORDER , CONTENT };
		for ( unsigned int* i = scol ; i != scol + 2 ; i++ ) {

			/* First reorder as per current row_order */
			reorder( idata[*i].begin()
				, idata[*i].end()
				, row_order.begin()
				, row_order.end()
			);

			/*
			 * For each equal range of previous sorts sub-sort this column
			 * and reorder the row_roder for that range accordingly
			 */
			vector< offset_range > newranges;
			for ( vector< offset_range >::const_iterator CJ = offsetranges.begin()
				; CJ != offsetranges.end()
				; CJ++
			){

				std::vector<strlist::difference_type> sub_order;
				
				sort_order( idata[*i].begin() + CJ->first 
					, idata[*i].begin() + CJ->second  
					, std::back_inserter(sub_order) 
				);

				reorder( row_order.begin() + CJ->first 
					, row_order.begin() + CJ->second 
					, sub_order.begin() 
					, sub_order.end() 
				);

				/* 
				 * Find equal_range's of this column within the equal_ranges defined by previous columns.
				 * Remeber, equal ranges in this column outside of these prior equal ranges 
				 * have been sorted according to a previous column and should not be re-sorted.
				 */
				
				equal_ranges( idata[*i].begin() + CJ->first ,  idata[*i].begin() + CJ->second , back_inserter(newranges) );
			}
			offsetranges = newranges;
		}

		/* re-order the rest of the columns */
		unsigned int ocol[] = { NAME , VVFLAGS  };
		for ( unsigned int* i = ocol ; i != ocol + 2 ; i++ ) {

			reorder( idata[*i].begin()
				, idata[*i].end()
				, row_order.begin()
				, row_order.end()
			);
		}
	}
		
	errstr << "Leaving exposure_specific_products" << Sas::message(Msg::App, Msg::Sparse);
	
}


HtmlPage RgsSummary::print() 
{
	HtmlPage page_("rgxpage");

	const HtmlTag hr("hr");

	std::string htitle("Observation " + PpssummCommon::obsid_ + " Pipeline Processing Summary");
	errstr << "Generating " << htitle << Sas::message(Msg::App, Msg::Verbose);

	/* Top of the page */ 
	page_.title( htitle );
	
	page_.linkbar("rgxpage");
	page_.body_ << hr ;
	
	page_.sub_title( PpssummCommon::sub_title_ , PpssummCommon::obsid_  + " RGS Processing Summary");
	page_.body_ << hr ;

	/* Middle of page */
	print_multi_exposure_products(page_);
	print_exposures_processed(page_);
	print_exposure_specific_products(page_);
	
	/* end of page */

	return page_;	

}

void RgsSummary::print_multi_exposure_products(HtmlPage& page)
{
	page.body_ << HtmlTag("h3" , "" , "RGS Multi-Exposure Products");
	if ( multi_table.size() ) {
		
		page.addTable(multi_title , multi_table);

	} else {
		page.body_ << HtmlTag("tt","","No multi-exposure products were produced.");
	}
	page.body_  << HtmlTag("hr");

}

void RgsSummary::print_exposures_processed(HtmlPage& page)
{
	/* Bad hack, copied from PpssummCommon */
	
        enum ExpColId { 
                INSTRUMENT =0 
                ,EXPID    
                ,SCHEDULED 
                ,MODE      
                ,DATA_MODE 
                ,FILTER   
                ,FILTER_WHEEL  // PN Uses column name FILTER_POTENTIOMETER
                ,DURATION
                ,ACTUAL_START
                ,ACTUAL_END  
                ,NUMCOL
        };


	page.body_ << HtmlTag("h3","","Exposures processed by PPS");

	if (exp_table.size() ) {

		exp_title = PpssummCommon::exposure_title_;
		exp_title.erase(exp_title.begin() + static_cast<strlist::difference_type>(FILTER_WHEEL));
		exp_title.erase(exp_title.begin() + static_cast<strlist::difference_type>(FILTER));

		page.addTable( exp_title , exp_table );

	} else {

		page.body_ << HtmlTag("tt","","No RGS exposures processed by the PPS.");

	}
	page.body_  << HtmlTag("hr");

}

void RgsSummary::print_exposure_specific_products(HtmlPage& page) 
{
	page.body_ << HtmlTag("h3","","Exposure Specific Products");

	if (exp_table.size() && specific_table.size() ) {

		/* Loop over exp_table so we get specific products in the same order */
		for (unsigned int i = 0 ; i < exp_table[0].size() ; i++ ) {

			std::string key =  exp_table[0][i] + exp_table[1][i];
			keyed_tables::const_iterator MI = specific_table.find( key );

			/* ignore exposures where there are no specific products */
			if ( MI == specific_table.end() ) continue;
	
			/* Copy the exposure information into a separate table for output */
			strtable exptmp(exp_table.size());
			for (unsigned int j = 0 ; j < exp_table.size() ; j++) {
				exptmp[j] = strlist(1 , exp_table[j][i]);
			}
			
			page.addTable( exp_title , exptmp , key );
			page.addTable( specific_title , MI->second );
			page.body_ << HtmlTag("hr");
		}

	} else {
		page.body_ << HtmlTag("tt","","No exposure specific products were produced.");
		page.body_  << HtmlTag("hr");

	}

}
