#ifndef PPSSUMM_GETSUMFILE
#define PPSSUMM_GETSUMFILE

#include <string>
#include <cstdlib>
#include <dirent.h>
#include "Tracer.h"
#include "errstr.h"


std::string getSumFile(std::string dirName="")
{
	/*
	 * Based on getSumFile from oal/.../Odf.cc where they declare it static
	 */

	if (! dirName.length() ) {
		
		char* p = getenv("SAS_ODF");
		if ( p == NULL ) {
			errstr << "SAS_ODF not defined."
				  << Sas::fatal("Sum2Fit:SAS_ODFUNDEFINED")
			;
		} else {
			dirName = p;
		}
	}
	
	errstr << "dirName " << dirName << Sas::message(Msg::App, Msg::Sparse);


        DIR *dirp = opendir(dirName.c_str());
        if (!dirp)
                errstr << "Cannot access directory given by SAS_ODF environment variable."
                          << Sas::fatal("Sum2Fit:DIRNOTFOUND")
		;

	std::string sumSetName;
	
        while (struct dirent *dp = readdir(dirp)) {
                string s(dp->d_name);
                if (s.length() >= 8)
                        s.erase(0, s.length()-7);

                if (s == "SUM.SAS") {
                        sumSetName = dp->d_name;
                        continue;
                } else if (s == "SUM.ASC") {
                        sumSetName = dp->d_name;
                        continue;
                }
        }
        closedir(dirp);

        if (sumSetName.empty()) {
		errstr << "Cannot find either a SUM.SAS or SUM.ASC format summary file"
			<< "In the directory given by SAS_ODF"
			<< Sas::fatal("Sum2Fit:SUMMARYFILENOTFOUND")
		;
		
	}
 
	errstr << "sumSetName " << sumSetName << Sas::message(Msg::App, Msg::Sparse);
        return (dirName + "/") + sumSetName;
}

#endif
