#include "string_algo.h"

std::string stripdir( std::string s )
{
	std::string::size_type cut = s.rfind('/');

	return ( cut == std::string::npos) 
		? s
		: s.substr( cut+1 , std::string::npos)
	;

}

std::string truncTime( std::string t ) 
{
	/* Given a string in dateTtime.nnn format strip off the .nnn */
	return t.substr( 0 , t.rfind(".") );
}


char toUpper(const char c)
{
	return static_cast<char>( toupper( static_cast<int>(c) ) );
}

char toLower(const char c)
{
	return static_cast<char>( tolower( static_cast<int>(c) ) );
}

std::string asHms (const std::string& s 
	, const int prec 
	, const double epsilon 
) {

	static const double sixty_ = 60.0;

	double d = strtod(s.c_str(),NULL);

	if ( errno == ERANGE )
	{
		errstr << "ppssumm::asHms_ Range error converting string to double." << Sas::fatal("ppssumm::string2double");	
	}

	double hr_t = fmod( ( d + 360.0 ), 360.0 ) / 15 ;
	double hr = int (hr_t);

	double min_t = sixty_ * hr_t - sixty_ * static_cast<double>(hr);
	double min = int (min_t);

	double sec = sixty_ * min_t - sixty_ * static_cast<double>(min);
	
	if ( sec == sixty_ ) sec -= epsilon;

	std::stringstream ss;
	if ( prec > 0 ) ss.precision(prec);
	ss << hr << "h" << min << "m" << sec << "s";
	return ss.str();

}

std::string asDms (const std::string& s 
	, int prec 
	, const double epsilon 
) {
	static const double sixty_ = 60.0;

	double d = strtod(s.c_str(),NULL);

	if ( errno == ERANGE )
	{
		errstr << "ppssumm::asDms_ Range error converting string to double." << Sas::fatal("ppssumm::string2double");	
	}

	int sgn = (d < 0) 
	? -1
	: 1
	;

	double deg_t = fabs (d);

	int deg = int(deg_t);

	double min_t = sixty_ * deg_t - sixty_ * static_cast<double>(deg);
	unsigned int min = int (min_t);

	double sec = sixty_ * min_t - sixty_ * static_cast<double>(min);

	if ( sec == sixty_ ) sec -= epsilon;

	std::stringstream ss;
	if ( prec > 0 ) ss.precision(prec);
	ss << (sgn * deg) << "d" << min << "m" << sec << "s";
	return ss.str();
}

